package com.ekexiu.project.platform.demand;

import com.ekexiu.project.platform.system.vo.SessionManager;
import com.ekexiu.project.util.HttpService;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.util.DateUtil;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.reflect.TypeReference;

import javax.sql.DataSource;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Path("/demand")
public class DemandService {
	private long jobDelay = 60;
	private String urlPrefix = "http://192.168.3.233:81/ajax/platform/sync";
	private boolean jobStarted = false;
	private String platformId = "F5DEA8EDB12E421A9FC4B701BB6176A6";
	private long qtime = 1000*60;
	
	private long ptime =0;

	@Autowrie("dataSource")
	private DataSource dataSource;

	@Autowrie
	private DemandDao demandDao;

	public DemandDao getDemandDao() {
		return demandDao;
	}

	public void setDemandDao(DemandDao demandDao) {
		this.demandDao = demandDao;
	}

	public long getJobDelay() {
		return jobDelay;
	}

	public void setJobDelay(long jobDelay) {
		this.jobDelay = jobDelay;
	}

	public String getUrlPrefix() {
		return urlPrefix;
	}

	public void setUrlPrefix(String urlPrefix) {
		this.urlPrefix = urlPrefix;
	}

	public DataSource getDataSource() {
		return dataSource;
	}

	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	public String getPlatformId() {
		return platformId;
	}

	public void setPlatformId(String platformId) {
		this.platformId = platformId;
	}

	@LoginUser
	@Put
	@Path("/modify")
	public int modify(@JdbcConn(true) Connection con, @RequestBody DemandInfo demand) throws SQLException, UnsupportedEncodingException {
		if (demandDao.update(con, demand) > 0) {

			DemandSyncLog dsl = new DemandSyncLog();
			dsl.setId(StringUtil.buildUUID());
			dsl.setUri("/update");
			StringBuilder sb = new StringBuilder();
			sb.append("pid=").append(this.platformId).append("&id=").append(demand.getId()).append("&title=")
					.append(URLEncoder.encode(demand.getTitle(), "UTF-8")).append("&descp=").append(URLEncoder.encode(demand.getDescp(), "UTF-8"))
					.append("&linkOrg=").append(URLEncoder.encode(demand.getLinkOrg(), "UTF-8")).append("&city=")
					.append(URLEncoder.encode(demand.getCityStr(), "UTF-8")).append("&invalidDay=").append(demand.getInvalidDay()).append("&cost=")
					.append(URLEncoder.encode(demand.getCostStr(), "UTF-8")).append("&duration=").append(URLEncoder.encode(demand.getDurationStr(), "UTF-8"));
			dsl.setParam(sb.toString());
			dsl.setTid(DateUtil.formatDateTime(System.currentTimeMillis()));
			demandDao.insert(con, dsl);
			return 1;
		}
		return 0;
	}

	@Post
	@Path("/delay")
	public int delay(@JdbcConn(true) Connection con, @LoginUser SessionManager user, String id, String day) throws SQLException {
		if (demandDao.update(con, id, user.getId(), day) > 0) {
			DemandSyncLog dsl = new DemandSyncLog();
			dsl.setId(StringUtil.buildUUID());
			dsl.setUri("/delay");
			StringBuilder sb = new StringBuilder();
			sb.append("pid=").append(this.platformId).append("&id=").append(id).append("&day=").append(day);
			dsl.setParam(sb.toString());
			dsl.setTid(DateUtil.formatDateTime(System.currentTimeMillis()));
			demandDao.insert(con, dsl);
			return 1;
		}
		return 0;
	}

	@LoginUser
	@Post
	@Path("/close")
	public int close(@JdbcConn(true) Connection con, String id) throws SQLException {
		if (demandDao.close(con, id, DateUtil.formatDateTime(System.currentTimeMillis())) > 0) {
			DemandSyncLog dsl = new DemandSyncLog();
			dsl.setId(StringUtil.buildUUID());
			dsl.setUri("/delete");
			StringBuilder sb = new StringBuilder();
			sb.append("pid=").append(this.platformId).append("&id=").append(id);
			dsl.setParam(sb.toString());
			dsl.setTid(DateUtil.formatDateTime(System.currentTimeMillis()));
			demandDao.insert(con, dsl);
			return 1;
		}
		return 0;
	}

	@LoginUser
	@Post
	@Path("/close/reason")
	public void closeReason(@JdbcConn(true)Connection con,String id,String[] reason)throws SQLException {
		demandDao.deleteReason(con, id);
		for (String s : reason) {
			DemandCloseReason dcr = new DemandCloseReason();
			dcr.setId(id);
			dcr.setDescp(s);
			demandDao.insert(con, dcr);
		}
	}

	private void syncPostKexiuData() throws Exception {
		Connection con = this.dataSource.getConnection();
		try {
			for (;;) {
				List<DemandSyncLog> list = demandDao.limitQuery(con, 1);
				if(list !=null && list.size()>0){
					DemandSyncLog dsl = list.get(0);
					HttpService.post(this.urlPrefix+dsl.getUri(), dsl.getParam(),null);
					demandDao.delete(con,dsl);
					con.commit();
				}else{
					return;
				}
			}
		} catch (Exception e) {
			con.rollback();
			throw e;
		} finally

		{
			con.close();
		}
	}
	
	private void syncGetKexiuData() throws Exception {
		Connection con = this.dataSource.getConnection();
		if(ptime ==0){
			String tstr = JdbcUtil.queryString(con, "SELECT MAX(ASS_TIME) FROM DEMAND_PROFESSOR","20180101010101");
			 java.text.SimpleDateFormat sdf = new SimpleDateFormat(DateUtil.DATETIME_FORMAT);
			this.ptime = sdf.parse(tstr).getTime();
		}
       long ctime = System.currentTimeMillis();
		try {
				
				String time = DateUtil.formatDateTime(this.ptime - this.qtime);
				String etime =DateUtil.formatDateTime(ctime);
				List<PlatformDemandProfessor> list = HttpService.get(this.urlPrefix+"/list?pid="+this.platformId+"&btime="+time+"&etime="+etime,new TypeReference<List<PlatformDemandProfessor>>() {
				}.getType());
				
				for(PlatformDemandProfessor pdf :list){
					if(this.demandDao.queryProfessor(con,pdf.getId(), pdf.getUid())==null){
						this.demandDao.insert(con,pdf);
					}
				}
				con.commit();
				this.ptime = ctime;
		} catch (Exception e) {
			con.rollback();
			throw e;
		} finally

		{
			con.close();
		}
	}

	@Get
	@Path("/jobStart")
	synchronized public void doJobStart() {

		if (!this.jobStarted) {
			this.jobStarted = true;
			JfwAppContext.getScheduledExecutorService().scheduleWithFixedDelay(new Runnable() {
				@Override
				public void run() {
					try {
						syncPostKexiuData();
						syncGetKexiuData();
					} catch (Throwable thr) {
					}
				}
			}, 0, this.jobDelay, TimeUnit.SECONDS);
		}

	}

	public class PlatformDemandProfessor extends DemandProfessor {
		private String pid;

		public String getPid() {
			return pid;
		}
		public void setPid(String pid) {
			this.setActived(false);
			this.pid = pid;
		}
	}

	@LoginUser
	@Path("/pq")
	@Get
	public PageQueryResult<Demand> pageQuery(@JdbcConn Connection con,String order, @Nullable String key,@Nullable Integer state,@Nullable Boolean actived,@Nullable String invalidDay,int pageSize,int pageNo)throws SQLException,JfwBaseException {
		if(order.equals("createTime")){
			return demandDao.pqByCreateTime(con, key == null ? null : "%" + key + "%", state, actived, invalidDay, pageSize, pageNo);
		} else if (order.equals("overTime")) {
			return demandDao.pqByOverTime(con, key == null ? null : "%" + key + "%", state, actived, invalidDay, pageSize, pageNo);
		} else if (order.equals("closeTime")) {
			return demandDao.pqByCloseTime(con, key == null ? null : "%" + key + "%", state, actived, invalidDay, pageSize, pageNo);
		} else {
			throw new JfwBaseException(700001, "need order");
		}
	}

	@LoginUser
	@Path("/pq/expired")
	public PageQueryResult<Demand> pageQueryExpired(@JdbcConn Connection con,String invalidDay,@Nullable String key,@Nullable Integer state,@Nullable Boolean actived,int pageSize,int pageNo)throws SQLException {
		return demandDao.pqInvalidDay(con, key, invalidDay, state, actived, pageSize, pageNo);
	}

	@LoginUser
	@Path("/qo")
	@Get
	public Demand query(@JdbcConn Connection con,String id)throws SQLException {
		return demandDao.query(con, id);
	}

	@LoginUser
	@Path("/closeReason")
	@Get
	public List<DemandCloseReason> queryReason(@JdbcConn Connection con, String id)throws SQLException {
		return demandDao.queryReason(con, id);
	}

	@LoginUser
	@Path("/pro/count")
	@Get
	public int proCount(@JdbcConn Connection con,String id)throws SQLException {
		return demandDao.proCount(con, id);
	}

	@LoginUser
	@Path("/pro")
	@Get
	public PageQueryResult<DemandProfessor> pageQuery(@JdbcConn Connection con,@Nullable Boolean actived, String id,int pageSize,int pageNo)throws SQLException {
		return demandDao.queryComments(con,actived, id, pageSize, pageNo);
	}

}
